/***************************************************************************
                          kcdlabel.cpp  -  description
                             -------------------
    begin                : jeu mar 23 20:03:45 CET 2000
    copyright            : (C) 2000 by Pascal 'PeP' Panneels
    email                : pp@bvdep.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kcdlabel.h"
#include "pixmaps.h"

KCDLabel::KCDLabel(const char *name) : KTopLevelWidget(name)
{
	//---------------------------------------------------
	// setup the background color to white and the title
	//---------------------------------------------------
	setBackgroundColor(white);
	setCaption("KCD-Label");
	setBackgroundMode ( NoBackground );

	//-------------------------------------------	
	// create a toolbar
	//-------------------------------------------
	toolBar()->insertButton( QPixmap(filenew), FILE_NEW, true, i18n("New Label") );
	toolBar()->insertButton( QPixmap(fileopen), FILE_LOAD, true, i18n("Open Existing Label") );
	toolBar()->insertButton( QPixmap(filesaveas), FILE_SAVE, true, i18n("Save Label") );
	toolBar()->insertSeparator();
	toolBar()->insertButton( QPixmap(fileprint), FILE_PRINT, true, i18n("Print Label") );
	toolBar()->insertSeparator();
	toolBar()->insertButton( QPixmap(eraser), OBJECT_DELETE, true, i18n("Delete Selection") );
	toolBar()->insertSeparator();
	toolBar()->insertButton( QPixmap(penguin), OBJECT_IMAGE, true, i18n("Insert Image") );
	toolBar()->insertButton( QPixmap(text), OBJECT_TEXT, true, i18n("Insert Text") );
	toolBar()->insertButton( QPixmap(centrejust), OBJECT_MULTI_TEXT, true, i18n("Insert Multi Line Text") );
	toolBar()->insertButton( QPixmap(rainbowtext), OBJECT_CURVED_TEXT, true, i18n("Insert Curved Text") );
	//toolBar()->enableMoving(false);
    QObject::connect( toolBar(), SIGNAL(clicked(int)), this, SLOT(slotToolBarClick(int)) );
    QObject::connect( toolBar(), SIGNAL(moved(BarPosition)), this, SLOT(slotToolBarMoved()));
    		
	//-------------------------------------------	
	// create a status bar
	//-------------------------------------------
	statusbar = new KStatusBar(this);
	setStatusBar( statusbar );
	statusbar->setInsertOrder(KStatusBar::RightToLeft);
	statusbar->insertItem( i18n("Page 0    "), STATUS_PAGE );
	statusbar->insertItem( "(1000,1000)", STATUS_MOUSEPOS);
	statusbar->insertItem( "KCDLabel ~ Pascal 'PeP' Panneels", STATUS_MSG );	
	statusbar->setAlignment( STATUS_PAGE, KStatusBar::Right);
	statusbar->setAlignment( STATUS_MOUSEPOS, KStatusBar::Right);
	
	//-------------------------------------------	
	// create the Popup menus and the main menu
	//-------------------------------------------
	filemenu = new QPopupMenu;
	filemenu->insertItem(i18n("&New"), this, SLOT(slotFileNew()));
	filemenu->insertItem(i18n("&Open..."), this, SLOT(slotFileOpen()));
	filemenu->insertItem(i18n("&Save..."), this, SLOT(slotFileSave()));
	filemenu->insertSeparator();
	filemenu->insertItem(i18n("&Print..."), this, SLOT(slotFilePrint()));
	//filemenu->insertItem(i18n("Print set&up..."), this, SLOT(slotFilePrintSetup()));
	filemenu->insertSeparator();
	filemenu->insertItem(i18n("&Quit"), qApp, SLOT(quit()));

	bodymenu = new QPopupMenu;
	bodymenu->insertItem(i18n("&Select source"), this, SLOT(slotBodySelectSource()));
	bodymenu->insertItem(i18n("&Read content"), this, SLOT(slotBodyReadCD()));
	bodymenu->insertSeparator();
		columnmenu = new QPopupMenu;
		columnmenu->insertItem(i18n("&None"), COLUMN_0);
		columnmenu->insertItem(i18n("&1"), COLUMN_1);
		columnmenu->insertItem(i18n("&2"), COLUMN_2);
		columnmenu->insertItem(i18n("&3"), COLUMN_3);
		columnmenu->insertItem(i18n("&4"), COLUMN_4);
		columnmenu->insertItem(i18n("&5"), COLUMN_5);
		columnmenu->insertItem(i18n("&6"), COLUMN_6);
		QObject::connect(columnmenu, SIGNAL(activated(int)), this, SLOT(slotNumberColumn(int)) );
		columnmenu->setItemChecked(COLUMN_0, true);
	bodymenu->insertItem(i18n("Number of &columns"), columnmenu);
		levelmenu = new QPopupMenu;
		levelmenu->insertItem(i18n("&1"), LEVEL_1);
		levelmenu->insertItem(i18n("&2"), LEVEL_2);
		levelmenu->insertItem(i18n("&3"), LEVEL_3);
		levelmenu->insertItem(i18n("&4"), LEVEL_4);
		levelmenu->insertItem(i18n("&All"), LEVEL_ALL);
		QObject::connect(levelmenu, SIGNAL(activated(int)), this, SLOT(slotNumberLevel(int)));
		levelmenu->setItemChecked(LEVEL_ALL, true);
	bodymenu->insertItem(i18n("Number of &directory levels"), levelmenu);
	bodymenu->insertItem(i18n("&Font to use"), this, SLOT(slotBodySelectFont()));
	bodymenu->insertSeparator();
	bodymenu->insertItem(i18n("&Lined"), BODY_LINE);
	bodymenu->setItemChecked(BODY_LINE, true);
	QObject::connect(bodymenu, SIGNAL(activated(int)), this, SLOT(slotBodyLined(int)) );
	bodymenu->insertItem(i18n("Draw &tree lines"), TREE_LINE);
	bodymenu->setItemChecked(TREE_LINE, true);
	QObject::connect(bodymenu, SIGNAL(activated(int)), this, SLOT(slotBodyDrawTree(int)) );
    bodymenu->insertItem(i18n("&Page number"), DISPLAY_PAGENUMBER);
	QObject::connect(bodymenu, SIGNAL(activated(int)), this, SLOT(slotBodyPageNumber(int)) );
	bodymenu->insertSeparator();
	bodymenu->insertItem(i18n("Body &layout"), this, SLOT(slotBodyLayout()));
								
	slidemenu = new QPopupMenu;
	slidemenu->insertItem(i18n("&Edit slide text"), this, SLOT(slotSlideEditText()));
    	alignmenu = new QPopupMenu;
    	alignmenu->insertItem(i18n("&Left"), ALIGN_LEFT);
    	alignmenu->insertItem(i18n("&Center"), ALIGN_CENTER);
    	alignmenu->insertItem(i18n("&Right"), ALIGN_RIGHT);
        alignmenu->setItemChecked(ALIGN_LEFT, true);
    	QObject::connect(alignmenu, SIGNAL(activated(int)), this, SLOT(slotAlignMenu(int)));
    slidemenu->insertItem(i18n("&Align"), alignmenu);
	slidemenu->insertItem(i18n("Transparent Background"), SLIDE_BACKGROUND);
	slidemenu->setItemChecked(SLIDE_BACKGROUND, false);
	QObject::connect(slidemenu, SIGNAL(activated(int)), this, SLOT(slotImageTransparent(int)) );

	imagemenu = new QPopupMenu;
	imagemenu->insertItem(i18n("Insert..."), this, SLOT(slotImageInsert()));
	imagemenu->insertItem(i18n("Delete"), this, SLOT(slotObjectDelete()));
	imagemenu->insertSeparator();
	imagemenu->insertItem(i18n("Resize to full page"), this, SLOT(slotImageResizeFullPage()));
	imagemenu->insertItem(i18n("Resize to ..."), this, SLOT(slotImageResize()));
	imagemenu->insertSeparator();
    	alignimagemenu = new QPopupMenu;
    	alignimagemenu->insertItem(i18n("&Center of page"), this, SLOT(slotImageAlignCenter()));
    imagemenu->insertItem(i18n("Align"), alignimagemenu);
    	orderimagemenu = new QPopupMenu;
    	orderimagemenu->insertItem(i18n("Increase"), this, SLOT(slotImageIncreaseOrder()));
		orderimagemenu->insertItem(i18n("Decrease"), this, SLOT(slotImageDecreaseOrder()));
    imagemenu->insertItem(i18n("Order..."), orderimagemenu);
    imagemenu->insertItem(i18n("Draw Tiled"), this, SLOT(slotImageTiled()));
    imagemenu->insertItem(i18n("Transparency"), this, SLOT(slotImageTransparency()));
    // set some ids
    imagemenu->setId(1, IMAGE_MENU_DELETE);
	imagemenu->setId(3, IMAGE_MENU_RESIZEFULL);
	imagemenu->setId(4, IMAGE_MENU_RESIZE);
	imagemenu->setId(6, IMAGE_MENU_ALIGN);
	imagemenu->setId(7, IMAGE_MENU_ORDER);
    imagemenu->setId(8, IMAGE_MENU_TILED);
    imagemenu->setId(9, IMAGE_MENU_TRANSPARENCY);
    QObject::connect(imagemenu, SIGNAL(activated(int)), this, SLOT(slotImageTiledMenu(int)) );
    imagemenu->setItemChecked(IMAGE_MENU_TRANSPARENCY, true);
        	
	stringmenu = new QPopupMenu;
	stringmenu->insertItem(i18n("Insert single line..."), this, SLOT(slotStringInsert()));
	stringmenu->insertItem(i18n("Insert curved single text ..."), this, SLOT(slotStringInsertCurved()));
	stringmenu->insertItem(i18n("Insert multi line text ..."), this, SLOT(slotStringInsertMulti()));
	stringmenu->insertItem(i18n("Delete"), this, SLOT(slotObjectDelete()));
    stringmenu->insertItem(i18n("Modify"), this, SLOT(slotStringModify()));
    stringmenu->insertSeparator();
    	orderstringmenu = new QPopupMenu;
    	orderstringmenu->insertItem(i18n("Increase"), this, SLOT(slotStringIncreaseOrder()));
		orderstringmenu->insertItem(i18n("Decrease"), this, SLOT(slotStringDecreaseOrder()));
    stringmenu->insertItem(i18n("Order..."), orderstringmenu);
    // set some ids
    stringmenu->setId(3, STRING_MENU_DELETE);
	stringmenu->setId(4, STRING_MENU_MODIFY);
	stringmenu->setId(6, STRING_MENU_ORDER);
    			
	pagemenu = new QPopupMenu;
	pagemenu->insertItem(i18n("Go to &front page"), this, SLOT(slotPageFront()));
	pagemenu->insertItem(i18n("Go to &back page"), this, SLOT(slotPageBack()));
	pagemenu->insertItem(i18n("Go to &page number..."), this, SLOT(slotPageNumber()));
	
	helpmenu = new QPopupMenu;
	helpmenu->insertItem(i18n("Manual"), this, SLOT(slotHelpManual()));
	helpmenu->insertItem(i18n("Thanks..."), this, SLOT(slotHelpThanks()));
	helpmenu->insertItem(i18n("About..."), this, SLOT(slotHelpAbout()));
	
	menuBar()->insertItem(i18n("&File"), filemenu);
	menuBar()->insertItem(i18n("&Body"), bodymenu);
	menuBar()->insertItem(i18n("&Slide"), slidemenu);
	menuBar()->insertItem(i18n("&Image"), imagemenu);
	menuBar()->insertItem(i18n("S&tring"), stringmenu);	
	menuBar()->insertItem(i18n("&Page"), pagemenu);
	menuBar()->insertSeparator();
	menuBar()->insertItem(i18n("&Help"), helpmenu);
	menuBar()->setMenuBarPos(KMenuBar::Top);
	menuBar()->enableFloating(false);
	menuBar()->enableMoving(false);
	
	//---------------------------------------------------
	// define popup menus activated by right click
	//---------------------------------------------------
	ImagePopupMenu = new QPopupMenu;
	ImagePopupMenu->insertItem(i18n("Resize to full page"), this, SLOT(slotImageResizeFullPage()));
	ImagePopupMenu->insertItem(i18n("Resize to ..."), this, SLOT(slotImageResize()));
	ImagePopupMenu->insertItem(i18n("Delete"), this, SLOT(slotObjectDelete()));
    	orderImagePopupMenu = new QPopupMenu;
    	orderImagePopupMenu->insertItem(i18n("Increase"), this, SLOT(slotImageIncreaseOrder()));
		orderImagePopupMenu->insertItem(i18n("Decrease"), this, SLOT(slotImageDecreaseOrder()));
 	ImagePopupMenu->insertItem(i18n("Order..."), orderImagePopupMenu);
    	alignImagePopupMenu = new QPopupMenu;
    	alignImagePopupMenu->insertItem(i18n("&Center"), this, SLOT(slotImageAlignCenter()));
 	ImagePopupMenu->insertItem(i18n("Align..."), alignImagePopupMenu);
    ImagePopupMenu->insertItem(i18n("Draw Tiled"), this, SLOT(slotImageTiled()));
    ImagePopupMenu->insertItem(i18n("Transparency"), this, SLOT(slotImageTransparency()));
    ImagePopupMenu->setId(5, IMAGE_MENU_TILED);
    ImagePopupMenu->setId(6, IMAGE_MENU_TRANSPARENCY);
	QObject::connect(ImagePopupMenu, SIGNAL(activated(int)), this, SLOT(slotImageTiledMenu(int)) );
	
	GlobalPopupMenu = new QPopupMenu;
	GlobalPopupMenu->insertItem(i18n("Insert image..."), this, SLOT(slotImageInsert()));
    GlobalPopupMenu->insertItem(i18n("Insert string..."), this, SLOT(slotStringInsert()));
	GlobalPopupMenu->insertItem(i18n("Insert curved string..."), this, SLOT(slotStringInsertCurved()));
	GlobalPopupMenu->insertItem(i18n("Insert multi line text ..."), this, SLOT(slotStringInsertMulti()));
	
    StringPopupMenu = new QPopupMenu;
    StringPopupMenu->insertItem(i18n("Delete"), this, SLOT(slotObjectDelete()));
    StringPopupMenu->insertItem(i18n("Modify"), this, SLOT(slotStringModify()));

	//---------------------------------------------------
	// setup the printing stuff
	//---------------------------------------------------
	Printing = false;
	Printer = new QPrinter;
  	Printer->setOrientation( QPrinter::Portrait );

	//---------------------------------------------------
	// setup misc stuff
	//---------------------------------------------------
  	ClickedPos = QPoint( -10000, -10000 );  	
  	CurrentPageToDisplay = 0;
  	Slide_Align = AlignLeft;
  	AlignChecked = ALIGN_LEFT;
	ObjectSelected = NULL;
	InitialPosForMove = QPoint(0,0);
	bufferpixmap.resize(size());
	SlideTransparentBackground=false;
	for (int i=0;i<MAX_PAGES;i++)
		FrontPages.push_back(KCoverPage());
		
	TheBody = new KCoverBody( this, "/mnt/cdrom", LEVEL_ALL, COLUMN_0, 2, true, true, false );
	
	//---------------------------------------------------
	// enable/disable some menu entries
	//---------------------------------------------------
	EnableComponents();
	
	//-------------------------------------------------------------------------
	// enable mouse tracking to adjust mouse position field in the status bar
	//-------------------------------------------------------------------------
	setMouseTracking(true);	
	
	//---------------------------------------------------
	// init qimageio extensions
	//---------------------------------------------------
	qInitImageIO();
  	
}

KCDLabel::~KCDLabel()
{
}

/* user press a key in the widget --> intercept page down/up and the like */
void KCDLabel::keyPressEvent ( QKeyEvent * e )
{
	switch ( e->key() )
		{
		case Key_Down :
		case Key_PageDown :
			ObjectSelected = NULL;
			CurrentPageToDisplay++;
			if (CurrentPageToDisplay==MAX_PAGES+1) CurrentPageToDisplay=MAX_PAGES;
			AdjustStatus();
			break;
		case Key_Up :
		case Key_PageUp :
			ObjectSelected = NULL;
			CurrentPageToDisplay--;		
			if (CurrentPageToDisplay==-1) CurrentPageToDisplay=0;
			AdjustStatus();
			break;
		case Key_Home :
			ObjectSelected = NULL;
			CurrentPageToDisplay = 0;
			AdjustStatus();
			break;
		case Key_End :
			ObjectSelected = NULL;
			CurrentPageToDisplay = MAX_PAGES;
			AdjustStatus();
			break;
		case Key_Delete :
			slotObjectDelete();
			break;
		case Key_Plus :
			if (ObjectSelected)
				switch ( ObjectSelected->GetClassType() )
					{
					case TYPE_CLASS_IMAGE :
						slotImageIncreaseOrder();
						break;
					case TYPE_CLASS_STRING :
					case TYPE_CLASS_CURVED_STRING :
						slotStringIncreaseOrder();
						break;
					case TYPE_CLASS_MULTI_STRING :
					    slotStringMultiIncreaseOrder();
					    break;
					}
			break;
		case Key_Minus :
			if (ObjectSelected)
				switch ( ObjectSelected->GetClassType() )
					{
					case TYPE_CLASS_IMAGE :
						slotImageDecreaseOrder();
						break;
					case TYPE_CLASS_STRING :
					case TYPE_CLASS_CURVED_STRING :
						slotStringDecreaseOrder();
						break;
					case TYPE_CLASS_MULTI_STRING :
					    slotStringMultiDecreaseOrder();
					    break;
					}
			break;
		}
	repaint();
}


/** Open a saved cd-label file */
void KCDLabel::slotFileOpen()
{
	char tmp[50];
	int err;
	int pagenumber;
	int version;
	
    QString Filename = QFileDialog::getOpenFileName(0, "*.kcd", this);
    if ( Filename.isEmpty() )
    	return;
	
    // open the fiel for reading
   	QFile file( Filename );
   	file.open( IO_ReadOnly );
   	
   	// read the signature and version
   	err = file.readBlock( tmp, 9 );
	if ( !strstr(tmp,TAG_SIGNATURE) || (err==-1) )
		return;	// bad file signature or file error
	version = tmp[8];

	// clear some parameters
	columnmenu->setItemChecked(TheBody->Columns, false);	
	levelmenu->setItemChecked( TheBody->Levels, false );
	// load the body
	TheBody->Load( file, version );
	TheBody->BuildResultTree();
			
	// read all pages
	err = file.readBlock( tmp, 1 );
	while ( err!=-1 && tmp[0] == TAG_PAGE )
		{
		err = file.readBlock( (char *)&pagenumber, sizeof(int) );
		if ( pagenumber == -1 )
			BackPage.Load( file, version );
		else
        	FrontPages[pagenumber].Load( file, version );
        tmp[0] = '*';
		err = file.readBlock( tmp, 1 );
		}	
	
	// close the file
	file.close();
	
	// adjust some menu lines according to the new body object
	AdjustBodyMenuParameters();
}

/** Save a cd-label into a file */
void KCDLabel::slotFileSave()
{
	char tmp[50];
	
    QString filename = QFileDialog::getSaveFileName(0, "*.kcd", this);
	QString Filename;
	
    if ( filename.isEmpty() )
    	return;
    	
	if ( !filename.contains(".kcd") )
		{
		Filename = filename + QString(".kcd");
		}
	else
		Filename = filename;
		
    // open the file for writing
   	QFile file( Filename );
   	file.open( IO_WriteOnly );
   	// write the signature and version
   	sprintf( tmp, "%s%c", TAG_SIGNATURE, TAG_VERSION );
   	file.writeBlock( tmp, 9 );
	
    // save the body
    TheBody->Save( file );
   	
   	// save the backpage
	BackPage.Save( file, -1 );
	// save the front pages
	for ( int i=0;i<MAX_PAGES;i++ )
		FrontPages[i].Save( file, i );
	
    // close the file --> it is finished !!		
	file.close();
}

/** Read the CD Content  */
void KCDLabel::slotBodyReadCD()
{
	QApplication::setOverrideCursor( waitCursor );	
	TheBody->myDirTree->Clear();	
	TheBody->myDirTree->SetBaseDir(TheBody->SourceCD);
	TheBody->myDirTree->Traverse();
	slotNumberLevel(TheBody->Levels);
	// restore normal cursor
	QApplication::restoreOverrideCursor();
}

/** Edit the text for the side of the CD */
void KCDLabel::slotSlideEditText()
{
	// open the slide edit dialog box
	SideEditDlg * dlg = new SideEditDlg(this, i18n("Slide Text"), &BackPage);
	dlg->exec();
}

/** go to the front page */
void KCDLabel::slotPageFront()
{
	CurrentPageToDisplay = 1;
	repaint();
}

/** go to back page */
void KCDLabel::slotPageBack()
{
	CurrentPageToDisplay = 0;
	repaint();
}

/** go to page number */
void KCDLabel::slotPageNumber()
{
	PageNumberDlg * dlg = new PageNumberDlg(this, i18n("Goto page..."), &CurrentPageToDisplay);
	dlg->exec();
	repaint();
}

/** Print the cd label */
void KCDLabel::slotFilePrint()
{
	if (Printer->setup())
		{
		int SavedTopX = TopX;
		int SavedTopY = TopY;
		TopX = 50;			// set top left point for drawings on the printer
		TopY = 50;
		Printing = true;
		QPainter paint;
		paint.begin( Printer );
		PaintBackPage(&paint);
		paint.translate( 0, BACK_V+1 );		// before it was (0, BACK_V+10)
		PaintFrontPage(&paint, 0);
		Printing = false;
		int curpage = 1;
		paint.translate(0,-BACK_V-1);			// before it was (0, -BACK_V-10)
		while ( FrontPages[curpage].NumberOfObjects || FrontPages[curpage].PageToPrint )
			{
			Printing = true;
			Printer->newPage();
			PaintFrontPage(&paint, curpage++ );
			paint.translate( 0, FRONT_V+1); 	// before it was (0, FRONT_V+10)
			PaintFrontPage(&paint, curpage++ );
			paint.translate( 0, -FRONT_V-1);	// before it was (0, -FRONT_V-10)		
			}
		Printer->newPage();
		paint.end();
		TopX = SavedTopX;
		TopY = SavedTopY;
		}
}

/** call the print setup dialog */
void KCDLabel::slotFilePrintSetup()
{
}

/** create a new cd label */
void KCDLabel::slotFileNew()
{
	// if any selected object, disable it
	ObjectSelected = NULL;
	EnableComponents();
	statusbar->changeItem( "", STATUS_MSG);	// clear the status bar message
	// erase all pages content
	BackPage.Clear();
	for (int i=0;i<MAX_PAGES;i++)
		FrontPages[i].Clear();
	slotNumberColumn(COLUMN_0);
	repaint();
}

/** Draw the back page of the cd cover */
void KCDLabel::PaintBackPage(QPainter * paint)
{
	// Erase previous drawing and draw the layout
    bufferpixmap.fill(white);

    paint->setPen( black );
    paint->drawLine( TopX, TopY, TopX+BACK_HI+BACK_HS*2, TopY );
    paint->drawLine( TopX, TopY+BACK_V, TopX+BACK_HI+BACK_HS*2, TopY+BACK_V );
    paint->drawLine( TopX, TopY, TopX, TopY+BACK_V );
    paint->drawLine( TopX+BACK_HI+BACK_HS*2, TopY, TopX+BACK_HI+BACK_HS*2, TopY+BACK_V );
	paint->drawLine( TopX+BACK_HS, TopY, TopX+BACK_HS, TopY+BACK_V );
    paint->drawLine( TopX+BACK_HS+BACK_HI, TopY, TopX+BACK_HS+BACK_HI, TopY+BACK_V );

   	QPoint orig(TopX, TopY);

    // draw the images
    if (BackPage.Images.size())
    	{
    	for (unsigned int i=0;i<BackPage.Images.size();i++)
    		{
    		KCoverImage * img = BackPage.GetImage(i);
			img->Paint( *paint, orig, BACK_HI, BACK_V, BACK_HS );
    		}
    	}

    // draw the texts
    if (BackPage.Strings.size())
    	{
    	for (unsigned int i=0;i<BackPage.Strings.size();i++)
    		{
    		KCoverString * str = BackPage.GetString(i);
    		str->Paint(*paint, orig );
    		}
    	}

    // draw the curved texts
    if (BackPage.CurvedStrings.size())
    	{
    	for (unsigned int i=0;i<BackPage.CurvedStrings.size();i++)
    		{
    		KCoverCurvedString * str = BackPage.GetCurvedString(i);
    		str->Paint(*paint, orig );
    		}
    	}
    	
    // draw the multi line texts
    if (BackPage.MultiStrings.size())
    	{
    	for (unsigned int i=0;i<BackPage.MultiStrings.size();i++)
    		{
    		KCoverMultiString * str = BackPage.GetMultiString(i);
    		str->Paint(*paint, orig );
    		}
    	}

    // display the side texts
    if (!SlideTransparentBackground)
    	{
    	paint->fillRect( TopX+1, TopY+1, BACK_HS-1, BACK_V-1, QBrush(*(BackPage.SideA->backcolor )));
    	paint->fillRect( TopX+BACK_HS+BACK_HI+1, TopY+1, BACK_HS-1, BACK_V-1, QBrush(*(BackPage.SideB->backcolor )));
    	}
    // Side A
    paint->setFont(*(BackPage.SideA->Font->Font));
    paint->translate( TopX, TopY+BACK_V );
    paint->rotate( -90 );
    paint->setPen(*(BackPage.SideA->textcolor));
    paint->drawText( 10, 0, BACK_V-10, BACK_HS, Slide_Align | AlignVCenter, BackPage.SideA->Text.c_str(), BackPage.SideA->Text.length() );
	paint->rotate( 90 );
	paint->translate( -TopX, -TopY-BACK_V );
	
    // Side B
    paint->setPen(*(BackPage.SideB->textcolor));
    paint->setFont(*(BackPage.SideB->Font->Font));
    //paint->translate( BACK_V, BACK_HS+BACK_HI );
    paint->translate( TopX+BACK_HI+2*BACK_HS, TopY );
    paint->rotate( 90 );
    paint->drawText( 10, BACK_HS-1, BACK_V-10, -BACK_HS, Slide_Align | AlignVCenter, BackPage.SideB->Text.c_str(), BackPage.SideB->Text.length() );

    // transform again to set all the other things normally !
    paint->rotate( -90 );
    //paint->translate( 0, BACK_HI );
    paint->translate( -TopX-BACK_HI-2*BACK_HS, -TopY );

    // draw the body content
    TheBody->Paint( paint, -1, TopX, TopY, BackPage );

}

/** Draw the back page of the cd cover */
void KCDLabel::PaintFrontPage(QPainter * paint, int page)
{
	// make page not to print now
	FrontPages[page].PageToPrint = false;
	
	// Erase previous drawing and draw the layout
    bufferpixmap.fill(white);

    paint->setPen( black );

    paint->drawLine( TopX, TopY, TopX+FRONT_H, TopY );
    paint->drawLine( TopX, TopY+FRONT_V, TopX+FRONT_H, TopY+FRONT_V );
    paint->drawLine( TopX, TopY, TopX, TopY+FRONT_V );
    paint->drawLine( TopX+FRONT_H, TopY, TopX+FRONT_H, TopY+FRONT_V );
	
   	QPoint orig(TopX, TopY);

    // draw the images
    if (FrontPages[page].Images.size())
    	{
    	for (unsigned int i=0;i<FrontPages[page].Images.size();i++)
    		{
    		KCoverImage * img = FrontPages[page].GetImage(i);
			img->Paint( *paint, orig, FRONT_H, FRONT_V );
    		}
    	}

    // draw the texts
    if (FrontPages[page].Strings.size())
    	{
    	for (unsigned int i=0;i<FrontPages[page].Strings.size();i++)
    		{
    		KCoverString * str = FrontPages[page].GetString(i);
    		str->Paint(*paint, orig );
    		}
    	}
    	
    // draw the curved texts
    if (FrontPages[page].CurvedStrings.size())
    	{
    	for (unsigned int i=0;i<FrontPages[page].CurvedStrings.size();i++)
    		{
    		KCoverCurvedString * str = FrontPages[page].GetCurvedString(i);
    		str->Paint(*paint, orig );
    		}
    	}

    // draw the multi line texts
    if (FrontPages[page].MultiStrings.size())
    	{
    	for (unsigned int i=0;i<FrontPages[page].MultiStrings.size();i++)
    		{
    		KCoverMultiString * str = FrontPages[page].GetMultiString(i);
    		str->Paint(*paint, orig );
    		}
    	}
	// paint the body    	
    TheBody->Paint( paint, page, TopX, TopY, FrontPages[page] );
}

/** define the paint event */
void KCDLabel::paintEvent( QPaintEvent * event)
{
	QPainter *paint = new QPainter(&bufferpixmap);
	int Hauteur;
	
	switch ( toolBar()->barPos() )
		{
		case KToolBar::Top :
			Hauteur = toolBar()->height();
			break;
		case KToolBar::Bottom :
			Hauteur = -toolBar()->height();
			break;
		default :
			Hauteur = 0;
			break;
		}
	
	switch ( CurrentPageToDisplay )
		{
		case 0 :
			TopX = ( width() - (BACK_HI+BACK_HS*2) ) / 2;
			TopY = menuBar()->height() + Hauteur + ( height() - menuBar()->height() - Hauteur - statusbar->height() - BACK_V ) / 2;
			PaintBackPage(paint);
			break;
		default :
			TopX = ( width() - FRONT_H ) / 2;
			TopY = menuBar()->height() + Hauteur + ( height() - menuBar()->height() - Hauteur - statusbar->height() - FRONT_V ) / 2;
			PaintFrontPage(paint, CurrentPageToDisplay-1);
			break;
		}
  	
  	if ( ClickedPos==QPoint(-10000, -10000) )
  		ClickedPos = QPoint( width()/2, height()/2 ) - QPoint( TopX, TopY );

    if ( ObjectSelected!=NULL )
    	{
    	if (ObjectSelected->IsMovable )
    		paint->setPen( green );
    	else
    		paint->setPen( red );
    	switch ( ObjectSelected->GetClassType() )
    		{
    		case TYPE_CLASS_IMAGE :
    			paint->drawRect(TopX+ObjectSelected->X-5, TopY+ObjectSelected->Y-5, ObjectSelected->Width()+10, ObjectSelected->Height()+10);
    			break;
    		case TYPE_CLASS_STRING  :
				{
				KCoverString * str = dynamic_cast<KCoverString *>(ObjectSelected);
				paint->drawRect(TopX+str->RealX()-10, TopY+str->RealY()-10, str->Width()+20, str->Height()+20);
				} break;
    		case TYPE_CLASS_CURVED_STRING  :
				{
				KCoverCurvedString * str = dynamic_cast<KCoverCurvedString *>(ObjectSelected);
				paint->drawRect(TopX+str->RealX()-10, TopY+str->RealY()-10, str->Width()+20, str->Height()+20 );
				} break;
    		case TYPE_CLASS_MULTI_STRING  :
				{
				KCoverMultiString * str = dynamic_cast<KCoverMultiString *>(ObjectSelected);
				QFontMetrics fm(*str->Font->Font);
				paint->drawRect(TopX+str->RealX()-10, TopY+str->RealY()-fm.height(), str->Width()+20, str->Height()+fm.height() );
				} break;
				
    		}
    	}
	bitBlt(this, event->rect().topLeft(), &bufferpixmap, event->rect());
	DisplayCurrentPage();
	delete paint;
}

/** display the current page in the status bar */
void KCDLabel::DisplayCurrentPage() const
{
	char buf[20];
	sprintf(buf,i18n("Page %d"), CurrentPageToDisplay);
	statusbar->changeItem( buf, STATUS_PAGE );
}

/** Select the side text alignement property */
void KCDLabel::slotAlignMenu(int id)
{
    alignmenu->setItemChecked(AlignChecked, false);
    alignmenu->setItemChecked(id, true);
	switch (id)
		{
		case ALIGN_LEFT :
			Slide_Align = AlignLeft;
			break;
		case ALIGN_CENTER :
			Slide_Align = AlignHCenter;
			break;
		case ALIGN_RIGHT :
			Slide_Align = AlignRight;
			break;
		}
	AlignChecked = id;
}

/** insert an image to the current page */
void KCDLabel::slotImageInsert()
{
    ImageFilename = QFileDialog::getOpenFileName(0, "*", this);
    if (ImageFilename.isEmpty())
    	return;
    	
    // add the image to the images list of the current displayed page
    if (CurrentPageToDisplay==0)	// this is the back page
		BackPage.AddImage( KCoverImage((const char *)(ImageFilename),0,0,0) );
    else
    	FrontPages[CurrentPageToDisplay-1].AddImage( KCoverImage((const char *)(ImageFilename),0,0,0) );
    repaint();
}

/** Resize an image to the full page size */
void KCDLabel::slotImageResizeFullPage()
{
	// if no object selected or object is not an image, return !
	if ( ObjectSelected==NULL || (ObjectSelected->GetClassType()!=TYPE_CLASS_IMAGE) ) return;
	
	ImageSelected = dynamic_cast<KCoverImage *>(ObjectSelected);
	// we are on backpage
	if (CurrentPageToDisplay==0)
		ImageSelected->Scale( BACK_HI+BACK_HS*2, BACK_V );
	else	// we are on a front page
		ImageSelected->Scale( FRONT_H, FRONT_V );
	repaint();
}

void KCDLabel::mouseDoubleClickEvent(QMouseEvent * event)
{
	// position of the drawn top left corner of the page	
	QPoint clickedpos = event->pos() - QPoint( TopX, TopY );

    if ( ObjectSelected!=NULL )
    	{
    	if (ObjectSelected->IsPointInObject(clickedpos))
    		{
   			ObjectSelected = NULL;
   			repaint();
   			EnableComponents();
   			statusbar->changeItem( "", STATUS_MSG);	// clear the status bar message
   			return;
    		}
    	}
    		
	if (CurrentPageToDisplay==0)
		ObjectSelected = BackPage.GetSelectedObject(clickedpos);
	else
		ObjectSelected = FrontPages[CurrentPageToDisplay-1].GetSelectedObject(clickedpos);	

	// set the status bar message
	AdjustStatus();
	repaint();
	EnableComponents();
}

void KCDLabel::mouseMoveEvent( QMouseEvent * event)
{
    QPoint pos = event->pos() - QPoint( TopX, TopY );

    // update the mouse pos status
    char buf[50];
    sprintf(buf,"(%d, %d)", pos.x(), pos.y());
	statusbar->changeItem(buf, STATUS_MOUSEPOS);
    	
	// if leftbutton not pressed --> return
	if ( event->state() != LeftButton )
		return;
			
    if ( InitialPosForMove==QPoint(0,0) )
    	{
    	InitialPosForMove = pos;
    	return;
    	}
    	
    if ( ObjectSelected!=NULL )
    	if ( ObjectSelected->IsMovable )
    		{
	    	//if (!ObjectSelected->IsPointInObject(pos))
    		//	return;
   			ObjectSelected->X += pos.x()-InitialPosForMove.x();
   			ObjectSelected->Y += pos.y()-InitialPosForMove.y();
   			InitialPosForMove = pos;
			AdjustStatus();
   			repaint();
    		}
}

/** mouse press event */
void KCDLabel::mousePressEvent( QMouseEvent * event )
{
	if (event->button()==RightButton )
		{
		ClickedPos = event->pos() - QPoint(TopX, TopY);
		if (ObjectSelected!=NULL)
			if ( ObjectSelected->GetClassType()==TYPE_CLASS_IMAGE )
				ImagePopupMenu->exec(QCursor::pos());
			else
				StringPopupMenu->exec(QCursor::pos());
		else
			GlobalPopupMenu->exec(QCursor::pos());
		}
}

/** mouse button released */
void KCDLabel::mouseReleaseEvent ( QMouseEvent * )
{
	InitialPosForMove = QPoint(0,0);
}

/** manage the resizing of the window */
void KCDLabel::resizeEvent(QResizeEvent * event)
{
	bufferpixmap.resize(event->size());
	KTopLevelWidget::resizeEvent(event);
}
/** make the slide background transparent */
void KCDLabel::slotImageTransparent(int id)
{
	if (id==SLIDE_BACKGROUND)
		{
		SlideTransparentBackground = !SlideTransparentBackground;
		slidemenu->setItemChecked(SLIDE_BACKGROUND, SlideTransparentBackground);
		}
}

/** align the image to the center of the page */
void KCDLabel::slotImageAlignCenter()
{
    if ( ObjectSelected!=NULL )
    	if ( ObjectSelected->GetClassType()==TYPE_CLASS_IMAGE )
    		{
    		ImageSelected = dynamic_cast<KCoverImage *>(ObjectSelected);
    		if (CurrentPageToDisplay == 0)
    			{
    			ImageSelected->X = ((BACK_HI+2*BACK_HS)/2) - (ImageSelected->Width()/2);
    			ImageSelected->Y = (BACK_V/2) - (ImageSelected->Height()/2);
    			}
    		else
    			{
    			ImageSelected->X = (FRONT_H/2) - (ImageSelected->Width()/2);
    			ImageSelected->Y = (FRONT_V/2) - (ImageSelected->Height()/2);
    			}
    		}
    repaint();
}

/** resize image to xxx % or pixels */
void KCDLabel::slotImageResize()
{
	if (ObjectSelected==NULL)
		return;
	if ( ObjectSelected->GetClassType()!=TYPE_CLASS_IMAGE )
		return;
	ScaleImageDlg * dlg = new ScaleImageDlg(this, i18n("Scale"), dynamic_cast<KCoverImage *>(ObjectSelected));
	
	dlg->exec();
}

/**  */
void KCDLabel::slotImageIncreaseOrder()
{
	if (!ObjectSelected) return;
	ObjectSelected->Order++;
	if (CurrentPageToDisplay==0)
		BackPage.SortImages();
	else
		FrontPages[CurrentPageToDisplay-1].SortImages();
	AdjustStatus();
}

/**  */
void KCDLabel::slotImageDecreaseOrder()
{
	if (!ObjectSelected) return;
	ObjectSelected->Order--;
	if (CurrentPageToDisplay==0)
		BackPage.SortImages();
	else
		FrontPages[CurrentPageToDisplay-1].SortImages();
	AdjustStatus();
}
/** display the about box */
void KCDLabel::slotHelpAbout()
{
	About * myabout = new About(this, i18n("About..."));
	myabout->exec();
}

/** insert a string on a page */
void KCDLabel::slotStringInsert()
{
	StringEditDlg * dlg;
	
	// open the slide edit dialog box
	if (CurrentPageToDisplay==0)
		dlg = new StringEditDlg(this, i18n("Insert a String"), &BackPage, NULL, ClickedPos);
	else
		dlg = new StringEditDlg(this, i18n("Insert a String"), &FrontPages[CurrentPageToDisplay-1], NULL, ClickedPos);
		
	dlg->exec();
}

/** delete selected string */
void KCDLabel::slotObjectDelete()
{
	if (ObjectSelected)
		{
		if (CurrentPageToDisplay==0)
			BackPage.DeleteSelectedObject(ObjectSelected);
		else
			FrontPages[CurrentPageToDisplay-1].DeleteSelectedObject(ObjectSelected);
		ObjectSelected = NULL;
		}
	EnableComponents();
	AdjustStatus();
}

/** modify the selected string */
void KCDLabel::slotStringModify()
{
	if (ObjectSelected==NULL)
		return;
	if ( ObjectSelected->GetClassType()!=TYPE_CLASS_STRING
	     && ObjectSelected->GetClassType()!=TYPE_CLASS_CURVED_STRING
	     && ObjectSelected->GetClassType()!=TYPE_CLASS_MULTI_STRING )
		return;
	
	StringEditDlg * dlg;
	StringMultiLineDlg * dlg2;
	
	// open the slide edit dialog box
	switch ( ObjectSelected->GetClassType() )
		{
		case TYPE_CLASS_STRING :
			{
			if (CurrentPageToDisplay==0)
				dlg = new StringEditDlg(this, i18n("Modify a String"), NULL, dynamic_cast<KCoverString *>(ObjectSelected), ClickedPos);
			else
				dlg = new StringEditDlg(this, i18n("Modify a String"), NULL, dynamic_cast<KCoverString *>(ObjectSelected), ClickedPos);
			dlg->exec();
			} break;
		case TYPE_CLASS_CURVED_STRING :
			{
			if (CurrentPageToDisplay==0)
				dlg = new StringCurvedEdit(this, i18n("Modify a Curved String"), NULL, dynamic_cast<KCoverCurvedString *>(ObjectSelected), ClickedPos);
			else
				dlg = new StringCurvedEdit(this, i18n("Modify a Curved String"), NULL, dynamic_cast<KCoverCurvedString *>(ObjectSelected), ClickedPos);
			dlg->exec();
			} break;
		case TYPE_CLASS_MULTI_STRING :
			{
			if (CurrentPageToDisplay==0)
				dlg2 = new StringMultiLineDlg(this, i18n("Modify a Curved String"), NULL, dynamic_cast<KCoverMultiString *>(ObjectSelected), ClickedPos);
			else
				dlg2 = new StringMultiLineDlg(this, i18n("Modify a Curved String"), NULL, dynamic_cast<KCoverMultiString *>(ObjectSelected), ClickedPos);
			dlg2->exec();
			} break;
		}
		
}

/**  */
void KCDLabel::slotStringIncreaseOrder()
{
	if (!ObjectSelected) return;
	ObjectSelected->Order++;
	if (CurrentPageToDisplay==0)
		BackPage.SortStrings();
	else
		FrontPages[CurrentPageToDisplay-1].SortStrings();
	AdjustStatus();
}

/**  */
void KCDLabel::slotStringDecreaseOrder()
{
	if (!ObjectSelected) return;
	ObjectSelected->Order--;
	if (CurrentPageToDisplay==0)
		BackPage.SortStrings();
	else
		FrontPages[CurrentPageToDisplay-1].SortStrings();
	AdjustStatus();
}

/** Enable/disable some controls according to the selection */
void KCDLabel::EnableComponents()
{
	// Disable/enable string entries
	stringmenu->setItemEnabled( STRING_MENU_DELETE, ObjectSelected && ( ObjectSelected->GetClassType()==TYPE_CLASS_STRING
			|| ObjectSelected->GetClassType()==TYPE_CLASS_CURVED_STRING ) );
	stringmenu->setItemEnabled( STRING_MENU_MODIFY, ObjectSelected && ( ObjectSelected->GetClassType()==TYPE_CLASS_STRING
			|| ObjectSelected->GetClassType()==TYPE_CLASS_CURVED_STRING ) );	
	stringmenu->setItemEnabled( STRING_MENU_ORDER, ObjectSelected && ( ObjectSelected->GetClassType()==TYPE_CLASS_STRING
			|| ObjectSelected->GetClassType()==TYPE_CLASS_CURVED_STRING ) );
				
	// Disable/enable images entries
	imagemenu->setItemEnabled( IMAGE_MENU_DELETE, ObjectSelected && ObjectSelected->GetClassType()==TYPE_CLASS_IMAGE );
	imagemenu->setItemEnabled( IMAGE_MENU_RESIZEFULL, ObjectSelected && ObjectSelected->GetClassType()==TYPE_CLASS_IMAGE );
	imagemenu->setItemEnabled( IMAGE_MENU_RESIZE, ObjectSelected && ObjectSelected->GetClassType()==TYPE_CLASS_IMAGE );
	imagemenu->setItemEnabled( IMAGE_MENU_ALIGN, ObjectSelected && ObjectSelected->GetClassType()==TYPE_CLASS_IMAGE );
	imagemenu->setItemEnabled( IMAGE_MENU_ORDER, ObjectSelected && ObjectSelected->GetClassType()==TYPE_CLASS_IMAGE );
	imagemenu->setItemEnabled( IMAGE_MENU_TILED, ObjectSelected && ObjectSelected->GetClassType()==TYPE_CLASS_IMAGE );
	imagemenu->setItemEnabled( IMAGE_MENU_TRANSPARENCY, ObjectSelected && ObjectSelected->GetClassType()==TYPE_CLASS_IMAGE );	
}

/** select the source dircetory to read from */
void KCDLabel::slotBodySelectSource()
{
	SourceSelectDlg * dlg = new SourceSelectDlg(this, "Source Directory",  TheBody->SourceCD);
	dlg->exec();
}

/** set the number of columns to display */
void KCDLabel::slotNumberColumn(int id)
{
	columnmenu->setItemChecked(TheBody->Columns, false);
	TheBody->Columns = id - COLUMN_0;
	columnmenu->setItemChecked(TheBody->Columns, true);	
}

/** fix the number of levels to scan */
void KCDLabel::slotNumberLevel(int id)
{
	levelmenu->setItemChecked( TheBody->Levels, false );
	TheBody->Levels = id;
	levelmenu->setItemChecked( TheBody->Levels, true );
	TheBody->BuildResultTree();
}

/** select font to use for body */
void KCDLabel::slotBodySelectFont()
{
	KFontDialog *fontdlg = new KFontDialog(this, i18n("Select Font"), true);
	QObject::connect(fontdlg, SIGNAL(fontSelected(const QFont &)), this, SLOT(slotBodyFontSelected(const QFont &)));
	fontdlg->setFont(*TheBody->BodyFont->Font);
	fontdlg->exec();
}

/** user select a font */
void KCDLabel::slotBodyFontSelected(const QFont &font)
{
	if (TheBody->BodyFont->Font)
		delete TheBody->BodyFont->Font;
	TheBody->BodyFont->Font = new QFont(font);
}

/** user select line or not */
void KCDLabel::slotBodyLined(int id)
{
	if (id==BODY_LINE)
		{
		TheBody->BodyLined = !TheBody->BodyLined;
		bodymenu->setItemChecked(BODY_LINE, TheBody->BodyLined );
		}
	
}
/** user select to draw or not the tree lines */
void KCDLabel::slotBodyDrawTree(int id)
{
	if (id==TREE_LINE)
		{
		TheBody->BodyTreeLine = !TheBody->BodyTreeLine;
		bodymenu->setItemChecked( TREE_LINE, TheBody->BodyTreeLine );
		}
}
/** enable page number display */
void KCDLabel::slotBodyPageNumber(int id)
{
	if (id==DISPLAY_PAGENUMBER)
		{
		TheBody->DisplayPageNumber = !TheBody->DisplayPageNumber;
		bodymenu->setItemChecked( DISPLAY_PAGENUMBER, TheBody->DisplayPageNumber);
		}
}

/** display body layout dialog */
void KCDLabel::slotBodyLayout()
{
	BodyLayout * dlg = new BodyLayout(this, i18n("Choose body layout"), &TheBody->BodyPage);
	dlg->exec();
}

/** invoke the help manual */
void KCDLabel::slotHelpManual()
{
	kapp->invokeHTMLHelp("kcdlabel/index.html", "");
}

/** display the correct status info according to selected object */
void KCDLabel::AdjustStatus()
{
	char buf[100];
	
	if ( ObjectSelected )
		sprintf(buf, i18n("Select.Obj. : Pos=(%d,%d), Order=%d"), ObjectSelected->X, ObjectSelected->Y, ObjectSelected->Order );
	else
		buf[0]=0;		
	
	statusbar->changeItem(buf, STATUS_MSG);
	
}

/** insert a line of text on a circle */
void KCDLabel::slotStringInsertCurved()
{
	StringCurvedEdit * dlg;
	
	// open the slide edit dialog box
	if (CurrentPageToDisplay==0)
		dlg = new StringCurvedEdit(this, i18n("Insert a Curved String"), &BackPage, NULL, ClickedPos);
	else
		dlg = new StringCurvedEdit(this, i18n("Insert a Curved String"), &FrontPages[CurrentPageToDisplay-1], NULL, ClickedPos);
	
	dlg->exec();
}

/** insert a multi line of text */
void KCDLabel::slotStringInsertMulti()
{
	StringMultiLineDlg * dlg;
	
	// open the slide edit dialog box
	if (CurrentPageToDisplay==0)
		dlg = new StringMultiLineDlg(this, i18n("Insert a Multi String"), &BackPage, NULL, ClickedPos);
	else
		dlg = new StringMultiLineDlg(this, i18n("Insert a Multi String"), &FrontPages[CurrentPageToDisplay-1], NULL, ClickedPos);
	
	dlg->exec();
}

void KCDLabel::slotStringMultiIncreaseOrder()
{
	if (!ObjectSelected) return;
	ObjectSelected->Order++;
	if (CurrentPageToDisplay==0)
		BackPage.SortMultiStrings();
	else
		FrontPages[CurrentPageToDisplay-1].SortMultiStrings();
	AdjustStatus();
}

/**  */
void KCDLabel::slotStringMultiDecreaseOrder()
{
	if (!ObjectSelected) return;
	ObjectSelected->Order--;
	if (CurrentPageToDisplay==0)
		BackPage.SortMultiStrings();
	else
		FrontPages[CurrentPageToDisplay-1].SortMultiStrings();
	AdjustStatus();
}

/** set the image to be drawn tiled */
void KCDLabel::slotImageTiled()
{
    if ( ObjectSelected!=NULL )
    	if ( ObjectSelected->GetClassType()==TYPE_CLASS_IMAGE )
    		{
    		KCoverImage * ImageSelected = dynamic_cast<KCoverImage *>(ObjectSelected);
			ImageSelected->ToggleTiled();
			}
}
/** check/uncheck the menu mark */
void KCDLabel::slotImageTiledMenu(int id)
{
	if ( ObjectSelected )
    	if ( ObjectSelected->GetClassType()==TYPE_CLASS_IMAGE )
    		{
    		KCoverImage * ImageSelected = dynamic_cast<KCoverImage *>(ObjectSelected);
			imagemenu->setItemChecked( id, ImageSelected->DrawTiled );
			ImagePopupMenu->setItemChecked( id, ImageSelected->DrawTiled );
			}
}

/** transparency of not */
void KCDLabel::slotImageTransparency()
{
    if ( ObjectSelected!=NULL )
    	if ( ObjectSelected->GetClassType()==TYPE_CLASS_IMAGE )
    		{
    		KCoverImage * ImageSelected = dynamic_cast<KCoverImage *>(ObjectSelected);
			ImageSelected->ToggleTransparency();
			}
}
/** user press a tool bar button */
void KCDLabel::slotToolBarClick(int id)
{
	switch ( id )
		{
		case FILE_NEW :
			slotFileNew();
			break;
		case FILE_LOAD :
			slotFileOpen();
			break;
		case FILE_SAVE :
			slotFileSave();
			break;
		case FILE_PRINT :
			slotFilePrint();
			break;
		case OBJECT_DELETE :
			slotObjectDelete();
			repaint();
			break;
		case OBJECT_IMAGE :
			slotImageInsert();
			break;
		case OBJECT_TEXT :
			slotStringInsert();
			break;
		case OBJECT_MULTI_TEXT :
			slotStringInsertMulti();
			break;
		case OBJECT_CURVED_TEXT :
			slotStringInsertCurved();
			break;
		}
}
/** user move the toolbar */
void KCDLabel::slotToolBarMoved()
{
	repaint();
}

/** adjust the body menu settings */
void KCDLabel::AdjustBodyMenuParameters()
{
	bodymenu->setItemChecked( BODY_LINE, TheBody->BodyLined );
	bodymenu->setItemChecked( TREE_LINE, TheBody->BodyTreeLine );
	bodymenu->setItemChecked( DISPLAY_PAGENUMBER, TheBody->DisplayPageNumber);
	columnmenu->setItemChecked(TheBody->Columns, true);	
	levelmenu->setItemChecked( TheBody->Levels, true );
}
/** open the greeting dialog */
void KCDLabel::slotHelpThanks()
{
	ThanksDlg * dlg = new ThanksDlg;
	dlg->exec();
}
