/***************************************************************************
                          kcoverpage.h  -  description
                             -------------------
    begin                : Sat Apr 15 2000
    copyright            : (C) 2000 by Pascal 'PeP' Panneels
    email                : pp@bvdep.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KCOVERPAGE_H
#define KCOVERPAGE_H

#include <kcoverobject.h>
#include <kcoverimage.h>
#include <kcoverstring.h>
#include <kcovercurvedstring.h>
#include <kcovermultistring.h>
#include <list.h>
#include <algo.h>
#include <qfile.h>

/**Base class for the front and the back pages
  *@author Pascal 'PeP' Panneels
  */

class KCoverPage {
public: 
  /** constructor / destructor */
  KCoverPage();
  virtual ~KCoverPage();
  	
  /** return a pointer to the clicked object */
  KCoverObject * GetSelectedObject(QPoint &);
  /** add an image to the list */
  void AddImage(KCoverImage img);
  /** return the image #i of the list */
  KCoverImage * GetImage(int i);
  /** Erase the selected object */
  void DeleteSelectedObject(KCoverObject *obj);
  /** Sort the image list */
  void SortImages();
  /** add a string to the page */
  void AddString(KCoverString str);
  /** return the string number i */
  KCoverString * GetString(int i);
  /** sort the string list */
  void SortStrings();
  /** clear the page content */
  void Clear();
  /** add a curved string to the page */
  void AddCurvedString(KCoverCurvedString str);
  /** return the curved string number i */
  KCoverCurvedString * GetCurvedString(int i);
  /** sort the curved string list */
  void SortCurvedStrings();
  /** add a multi string to the page */
  void AddMultiString(KCoverMultiString str);
  /** return the multi string number i */
  KCoverMultiString * GetMultiString(int i);
  /** sort the multi string list */
  void SortMultiStrings();
  /** save the page */
  virtual void Save(QFile &file, int pagenumber);
  /** load a page */
  virtual void Load(QFile &file, int version);
  	
public :
	std::list<KCoverImage> Images;
	std::list<KCoverString> Strings;
	std::list<KCoverCurvedString> CurvedStrings;
	std::list<KCoverMultiString> MultiStrings;
	int NumberOfObjects;
	bool PageToPrint;
};

#endif
